####################################################################
# Utility functions
#
# Copyright (C) 2002, Matthias Baas (baas@ira.uka.de)
#
# http://cgkit.sourceforge.net
#
# You may distribute under the terms of the BSD license, as
# specified in the file license.txt.
####################################################################

import types
from cgtypes import vec3
from ri import *
try:
    import Image
    _PIL_installed = 1
except ImportError:
    _PIL_installed = 0


def RiuArrow(height=1.0, thickness=0.1, headheight=0.2, headscale=1.7):
    """Creates an arrow.

    The arrow starts in the origin and ends in (0,0,height). thickness is the
    overall thickness of the arrow, headheight and headscale specify the
    height and a factor for the radius of the arrow head.
    """
    
    # Radius for arrow head (paraboloid)
    rmax = headscale*thickness
    a = headheight/(rmax*rmax)
    offset = a*thickness*thickness
    RiTransformBegin()
    RiCylinder(thickness, 0,height-0.01-offset, 360)
    RiTranslate(0,0,height)
#    RiRotate(180,1,0,0)
    RiParaboloid(rmax, 0,-headheight,360)
    RiTransformEnd()

def RiuCoordSystem(thickness=0.06):
    """Create a coordinate system."""
    
    RiAttributeBegin()
    RiSurface("matte")
    # X axis
    RiColor((1,0,0))
    RiTransformBegin()
    RiRotate(90, 0,1,0)
    RiuArrow(thickness=thickness)
    RiTransformEnd()

    # Y axis
    RiColor((0,1,0))
    RiTransformBegin()
    RiRotate(-90, 1,0,0)
    RiuArrow(thickness=thickness)
    RiTransformEnd()

    # Z axis
    RiColor((0,0,1))
    RiuArrow(thickness=thickness)
    
    RiAttributeEnd()

def RiuGrid(thickness=0.02):
    cells = 6
    
    RiAttributeBegin()
    RiSurface("matte")
    RiColor((0.9,0.9,0.9))

    RiTransformBegin()
    RiRotate(-90,1,0,0)
    RiTranslate(-cells/2,0,-cells/2)
    for i in range(cells+1):
        RiCylinder(thickness,0,cells,360)
        RiTranslate(1,0,0)
    RiTransformEnd()

    RiTransformBegin()
    RiRotate(90,0,1,0)
    RiTranslate(0,-cells/2,-cells/2)
    for i in range(cells+1):
        RiCylinder(thickness,0,cells,360)
        RiTranslate(0,1,0)
    RiTransformEnd()

    RiAttributeEnd()


def RiuHeightfield(image):
    if not _PIL_installed:
        raise ImportError, "the Python Imaging Library (PIL) is not installed"
    
    if type(image)==types.StringType or type(image)==types.UnicodeType:
        image = Image.open(image)

    points = []
    width, height = image.size
    for j in range(height):
        y = float(j)/(height-1)
        for i in range(width):
            x = float(i)/(height-1)
            col=image.getpixel((i,j))
            z=col[0]/255.0
            p=vec3(x,y,z)
            points.append(p)

    RiPatchMesh(RI_BILINEAR, width, RI_NONPERIODIC, height, RI_NONPERIODIC, P=points)


##########################################################

if __name__=='__main__':
    pass
