####################################################################
# Shading Language functionality
#
# Copyright (C) 2002, Matthias Baas (baas@ira.uka.de)
#
# http://cgkit.sourceforge.net
#
# You may distribute under the terms of the BSD license, as
# specified in the file license.txt.
####################################################################

import math, random, string, re
from math import acos, asin, ceil, cos, exp, floor, pow, sin, sqrt, tan
import noise
from cgtypes import vec3 as _vec3

# Builtin functions
abs = abs
max = max
min = min
round = round

PI = math.pi

def _tovec3(arg):
    try:
        a = len(arg)
    except:
        a = 1
    if a==1:
        return _vec3(arg,0.0,0.0)
    elif a==2:
        x,y=arg
        return _vec3(x,y,0.0)
    elif a==3:
        x,y,z=arg
        return _vec3(x,y,z)
    elif a==4:
        x,y,z,t=arg
        return _vec3(x,y,z)
    else:
        return _vec3()
    

def atan(*args):
    if len(args)==1:
        return math.atan(args[0])
    elif len(args)==2:
        return math.atan2(args[0],args[1])
    else:
        raise TypeError,"atan only takes 1 or 2 arguments."

def log(*args):
    if len(args)==1:
        return math.log(args[0])
    elif len(args)==2:
        return math.log(args[0])/math.log(args[1])
    else:
        raise TypeError,"log only takes 1 or 2 arguments."
    

# clamp
def clamp(a, amin, amax):
    return min(max(a,amin), amax)

# degrees
def degrees(rad):
    """Convert from radians to degrees."""
    return rad*180.0/PI

# radians
def radians(deg):
    """Convert from degrees to radians."""
    return deg*PI/180.0

def Du(p):
    pass

def Dv(p):
    pass

def Deriv(num, den):
    pass

def filterstep(edge, s1):
    pass

def inversesqrt(x):
    return 1.0/sqrt(x)

def mix(c0,c1,value):
    return (1.0-value)*c0 + value*c1

def mod(a,b):
    return a%b

def float_noise(*args):
    la = len(args)
    if la==1:
        return noise.noise(args[0])
    elif la==2:
        return noise.noise(args[0],args[1])
    elif la==3:
        return noise.noise(args[0],args[1],args[2])
    elif la==4:
        return noise.noise(args[0],args[1],args[2],args[3])
    else:
        raise TypeError, "the function takes between 1 and 4 arguments ("+`la`+" given)"


def point_noise(*args):
    la = len(args)
    if la==1:
        try:
            a = len(args[0])
        except:
            a = 1
        if a==1:
            return noise.vnoise(args[0],0,0)
        elif a==2:
            return noise.vnoise(args[0],0)
        elif a==3:
            return noise.vnoise(args[0])
        else:
            raise ValueError,"arg1: invalid argument length"            
    elif la==2:
        try:
            a = len(args[0])
        except:
            a = 1
        if a==1:
            return noise.vnoise(args[0],args[1],0)
        elif a==3:
            return noise.vnoise(args[0],args[1])
        else:
            raise ValueError,"arg1: invalid argument length"
    elif la==3:
        return noise.vnoise(args[0],args[1],args[2])
    elif la==4:
        x,y,z,t = noise.vnoise(args[0],args[1],args[2],args[3])
        return _vec3(x,y,z)
    else:
        raise TypeError, "the function takes between 1 and 4 arguments ("+`la`+" given)"
   
color_noise = point_noise
vector_noise = point_noise

def float_pnoise(*args):
    la = len(args)
    try:
        a = len(args[0])
    except:
        a = 1
        
    if la==2:
        if a==1:
            return noise.pnoise((args[0],),(args[1],))
        else:
            return noise.pnoise(args[0],args[1])
    elif la==4:
        if a==1:
            return noise.pnoise((args[0],args[1]),(args[2],args[3]))
        else:
            return noise.pnoise(args[0],args[1],args[2],args[3])
    else:
        raise TypeError, "the function takes between 1 and 4 arguments ("+`la`+" given)"

def point_pnoise(*args):
    la = len(args)
    try:
        a = len(args[0])
    except:
        a = 1
        
    if la==2:
        if a==1:
            res = noise.vpnoise((args[0],),(args[1],))
        else:
            res = noise.vpnoise(args[0],args[1])
    elif la==4:
        if a==1:
            res = noise.vpnoise((args[0],args[1]),(args[2],args[3]))
        else:
            res = noise.vpnoise(args[0],args[1],args[2],args[3])
    else:
        raise TypeError, "the function takes between 1 and 4 arguments ("+`la`+" given)"

    return _tovec3(res)

color_pnoise = point_pnoise
vector_pnoise = point_pnoise

def float_cellnoise(*args):
    la = len(args)
    if la==1:
        return noise.cellnoise(args[0])
    elif la==2:
        return noise.cellnoise(args[0],args[1])
    elif la==3:
        return noise.cellnoise(args[0],args[1],args[2])
    elif la==4:
        return noise.cellnoise(args[0],args[1],args[2],args[3])
    else:
        raise TypeError, "the function takes between 1 and 4 arguments ("+`la`+" given)"

def point_cellnoise(*args):
    la = len(args)
    if la==1:
        try:
            a = len(args[0])
        except:
            a = 1
        if a==1:
            return noise.vcellnoise(args[0],0,0)
        elif a==2:
            return noise.vcellnoise(args[0],0)
        elif a==3:
            return noise.vcellnoise(args[0])
        else:
            raise ValueError,"arg1: invalid argument length"            
    elif la==2:
        try:
            a = len(args[0])
        except:
            a = 1
        if a==1:
            return noise.vcellnoise(args[0],args[1],0)
        elif a==3:
            return noise.vcellnoise(args[0],args[1])
        else:
            raise ValueError,"arg1: invalid argument length"
    elif la==3:
        return noise.vcellnoise(args[0],args[1],args[2])
    elif la==4:
        x,y,z,t = noise.vcellnoise(args[0],args[1],args[2],args[3])
        return _vec3(x,y,z)
    else:
        raise TypeError, "the function takes between 1 and 4 arguments ("+`la`+" given)"
   
color_cellnoise = point_cellnoise
vector_cellnoise = point_cellnoise

def float_random():
    return random.random()

def color_random():
    return _vec3(random.random(), random.random(), random.random())

def point_random():
    return _vec3(random.random(), random.random(), random.random())

def sign(x):
    if x<0:
        return -1
    elif x>0:
        return 1
    else:
        return 0

def smoothstep(min, max, x):
    if x<min:
        return 0.0
    if x>max:
        return 1.0
    x = (x-min)/(max-min)
    return x*x*(3.0-2.0*x)

def spline(x, knots):

    nknots = len(knots)
    nspans = nknots-3
 
    if nspans<1:
        raise ValueError, "spline(): there must be at least 4 control points ("+`nknots`+" given)"

    x = clamp(x, 0.0, 1.0)*nspans
    span = int(x)
    if span>=nknots-3:
        span = nknots-4
    x    -= span
    knot0, knot1, knot2, knot3 = knots[span:span+4]

    c3 = -0.5*knot0 + 1.5*knot1 - 1.5*knot2 + 0.5*knot3
    c2 =      knot0 - 2.5*knot1 + 2.0*knot2 - 0.5*knot3
    c1 = -0.5*knot0 + 0.5*knot2
    c0 =      knot1

    return ((c3*x + c2)*x + c1)*x + c0


def step(min, x):
    if x<min:
        return 0.0
    else:
        return 1.0


def distance(P1,P2):
    return (P2-P1).length()

def faceforward(N,I,Nref):
    return sign(-I*Nref)*N

def length(v):
    return v.length()

def normalize(v):
    return v.normalize()

def reflect(I, N):
    return I.reflect(N)

def refract(I, N, eta):
    return I.refract(N,eta)

def xcomp(P):
    return P.x

def ycomp(P):
    return P.y

def zcomp(P):
    return P.z

def setxcomp(P,x):
    P.x = x

def setycomp(P,y):
    P.y = y

def setzcomp(P,z):
    P.z = z

def comp(c, index):
    return c[index]

def setcomp(c, index, value):
    c[index]=value

def concat(*args):
    return string.join(args,"")
    
def match(pattern, subject):
    return re.search(pattern, subject)!=None

def format(pattern, *args, **keyargs):
    res=""
    while 1:
        n=pattern.find("%")
        if n==-1:
            res+=pattern
            break
        res+=pattern[0:n]+"%s"
        pattern=pattern[n+2:]

    if keyargs.has_key("args"):
        args=keyargs["args"]

    return res % args

def printf(pattern, *args):
    print format(pattern,args=args)

######################################################################

if __name__=="__main__":

    printf("Hallo s=%f c=%c",0.5,(1,2,3))
    
